@@ -60,7 +60,7 @@ class FilePickerTests(FileUploadTestCase):
         Very basic smoke test for file picker.
 
         """
-        response = self.client.get('/adminfiles/?field=test')
+        response = self.client.get('/adminfiles/all/?field=test')
         self.assertContains(response, 'href="/media/adminfiles/tiny.png"')
         self.assertContains(response, 'href="/media/adminfiles/somefile.txt')
             
@@ -73,12 +73,21 @@ class FilePickerTests(FileUploadTestCase):
         response = self.client.get('/adminfiles/files/?field=test')
         self.assertNotContains(response, 'href="/media/adminfiles/tiny.png"')
         self.assertContains(response, 'href="/media/adminfiles/somefile.txt')
+        
+    def test_custom_links(self):
+        _old_links = settings.ADMINFILES_INSERT_LINKS.copy()
+        settings.ADMINFILES_INSERT_LINKS['text/plain'] = [('Crazy insert', {'yo': 'thing'})]
+       
+        response = self.client.get('/adminfiles/all/?field=test')
+        self.assertContains(response, 'rel="some-file:yo=thing"')
+        
+        settings.ADMINFILES_INSERT_LINKS = _old_links
 
     def test_thumb_order(self):
         _old_order = settings.ADMINFILES_THUMB_ORDER
         settings.ADMINFILES_THUMB_ORDER = ('title',)
         
-        response = self.client.get('/adminfiles/?field=test')
+        response = self.client.get('/adminfiles/all/?field=test')
         image_index = response.content.find('tiny.png')
         file_index = response.content.find('somefile.txt')
         self.failUnless(image_index > 0)
@@ -176,7 +185,18 @@ class RenderTests(TemplateTestCase, FileUploadTestCase):
     
     def test_render_template_used(self):
         render_uploads('<<<some-file>>>')
-        self.assertEquals(self.templates[0].name, 'adminfiles/render.html')
+        self.assertEquals(self.templates[0].name,
+                          'adminfiles/render/default.html')
+
+    def test_render_mimetype_template_used(self):
+        render_uploads('<<<an-image>>>')
+        self.assertEquals(self.templates[0].name,
+                          'adminfiles/render/image/default.html')
+
+    def test_render_subtype_template_used(self):
+        render_uploads('<<<an-image>>>', 'alt')
+        self.assertEquals(self.templates[0].name,
+                          'alt/image/png.html')
 
     def test_render_whitespace(self):
         render_uploads('<<< some-file \n>>>')
@@ -240,7 +260,7 @@ class RenderTests(TemplateTestCase, FileUploadTestCase):
         self.failUnless('<a href="/media/adminfiles/somefile.txt"' in html)
         
     def test_default_template_renders_link_class(self):
-        html = render_uploads('<<<some-file:class=other classes>>>')
+        html = render_uploads(u'<<<some-file:class=other classes>>>')
         self.failUnless('class="other classes"' in html)
 
     def test_default_template_renders_link_title(self):
@@ -251,21 +271,63 @@ class RenderTests(TemplateTestCase, FileUploadTestCase):
         html = render_uploads('<<<some-file:title=Other name>>>')
         self.failUnless('Other name' in html)
 
-    def test_render_template_filter(self):
-        tpl = template.Template('{% load adminfiles_tags %}'
-                                '{{ post.content|render_uploads|safe }}')
+    def test_template_override(self):
+        html = render_uploads('<<<an-image:as=default>>>')
+        self.failUnless('<a href="/media/adminfiles/tiny.png"' in html)
+
+    def test_template_override_fallback(self):
+        html = render_uploads('<<<some-file:as=image/jpeg>>>')
+        self.failUnless('<img src="/media/adminfiles/somefile.txt"' in html)
+
+    def test_template_override_with_nonexisting(self):
+        html = render_uploads('<<<an-image:as=some/wonky>>>')
+        self.failUnless('<a href="/media/adminfiles/tiny.png"' in html)
+
+    def test_render_uploads_template_filter(self):
+        tpl = template.Template(u'{% load adminfiles_tags %}'
+                                u'{{ post.content|render_uploads|safe }}')
         html = tpl.render(template.Context({
-                    'post': Post(title='a post',
-                                 content='<<<some-file>>>')}))
-        self.assertEquals(self.templates[1].name, 'adminfiles/render.html')
+                    'post': Post(title=u'a post',
+                                 content=u'<<<some-file>>>')}))
+        self.assertEquals(self.templates[1].name,
+                          'adminfiles/render/default.html')
         self.failUnless('<a href' in html)
 
-    def test_render_template_filter_alt_template(self):
+    def test_render_uploads_template_filter_alt_template(self):
         tpl = template.Template(
-            '{% load adminfiles_tags %}'
-            '{{ post.content|render_uploads:"other.html" }}')
+            u'{% load adminfiles_tags %}'
+            u'{{ post.content|render_uploads:"alt" }}')
         html = tpl.render(template.Context({
-                    'post': Post(title='a post',
-                                 content='<<<some-file>>>')}))
-        self.assertEquals(self.templates[1].name, 'other.html')
+                    'post': Post(title=u'a post',
+                                 content=u'<<<some-file>>>')}))
+        self.assertEquals(self.templates[1].name, 'alt/default.html')
+        
+    def test_render_upload_template_filter(self):
+        tpl = template.Template(u'{% load adminfiles_tags %}'
+                                u'{{ img|render_upload }}')
+        html = tpl.render(template.Context({'img': self.animage}))
+        self.assertEquals(self.templates[1].name,
+                          'adminfiles/render/image/default.html')
+        self.failUnless('<img src' in html)
+
+    def test_render_upload_template_filter_options(self):
+        tpl = template.Template('{% load adminfiles_tags %}'
+                                '{{ img|render_upload:"alt=blah" }}')
+        html = tpl.render(template.Context({'img': self.animage}))
+        self.failUnless('alt="blah"' in html)
+
+    def test_render_upload_template_filter_alt_template(self):
+        tpl = template.Template(
+            u'{% load adminfiles_tags %}'
+            u'{{ f|render_upload:"template_path=alt" }}')
+        html = tpl.render(template.Context({'f': self.somefile}))
+        self.assertEquals(self.templates[1].name, 'alt/default.html')
+        
+    def test_render_upload_template_filter_alt_template_options(self):
+        tpl = template.Template(
+            u'{% load adminfiles_tags %}'
+            u'{{ f|render_upload:"template_path=alt:class=yo" }}')
+        html = tpl.render(template.Context({'f': self.somefile}))
+        self.assertEquals(self.templates[1].name, 'alt/default.html')
+        self.failUnless('class="yo"' in html)
         
